#\file actHelpers.py
#
#----------------------------------------------------------------------------------
#
#   Gathers info from ANSYS and manipulates results
#   
#----------------------------------------------------------------------------------
#----------------------------------------------------------------------------------
#
# author: Tone Wermundsen
# Last update: 18.12.2014
# ansys v15 ACT 
#
#-----------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------
#	gets principal or equivalent values for nodes
#-----------------------------------------------------------------------------------
def getNodeValues(compT1, compT2, Stress, stressOption):
    resultSet = {}
    for key in compT2:
        resultSet[key] = []
        for item in xrange(len(compT2[key])):
            try:
                compT2[key][item] = map(operator.sub, compT2[key][item], compT1[key][item])
            except:
                compT2[key][item] = compT2[key][item]   

        x = compT2[key][0]
        y = compT2[key][1]
        z = compT2[key][2]
        xy = compT2[key][3]
        xz = compT2[key][4]
        yz = compT2[key][5]

        main1 = []; main2 = []; main3 = []; equv = []
        app1 = main1.append
        app2 = main2.append
        app3 = main3.append
        if stressOption != "Equivalent Stress":
            for (x,y,z,xy,xz,yz) in zip(x,y,z,xy,xz,yz):
                if Stress == False:
                    principal = eigenValue_FatigueDNV(x,y,z,0.5*xy,0.5*xz,0.5*yz)
                elif Stress == True and stressOption == "Absolut Maximal Principal Stress": 
                    principal = eigenValue_FatigueDNV(x,y,z,xy,xz,yz)
                app1(principal[int('1')])
                app2(principal[int('2')])
                app3(principal[int('3')])
            main1 = map(abs, main1)
            main2 = map(abs, main2)
            main3 = map(abs, main3)
            for i in xrange(len(main1)):
                resultSet[key].append(main1[i])#(max(main1[i], main2[i], main3[i]))
        elif Stress == True and stressOption == "Equivalent Stress":   
            for (x,y,z,xy,xz,yz) in zip(x,y,z,xy,xz,yz):
                equvNum = math.sqrt(0.5*((x-y)**2 + (y-z)**2 + (z-x)**2) + 3.*(xy**2 + yz**2 + xz**2))
                equv.append(equvNum)
            resultSet[key] = equv
    return resultSet

#-----------------------------------------------------------------------------------
#	gets combined components values for nodes
#-----------------------------------------------------------------------------------
def gatherNodeComponents(mesh, elements, resultItem, time, lc):
    nodeComponents = {}
    for elementId in elements:
        element = mesh.ElementById(elementId)
        cornerNodes = element.CornerNodeIds
        for Id in cornerNodes:
            node = mesh.NodeById(Id)
            if nodeComponents.has_key((lc, Id, elementId, time)):
                pass
            else:
                nodeComponents[lc, Id, elementId, time] = getNodalComponentsFromOneElement(resultItem, mesh, Id, elementId)
    return nodeComponents

#-----------------------------------------------------------------------------------
#	 gets all components values for nodes
#-----------------------------------------------------------------------------------
def getNodalComponentsFromOneElement(resultItem, mesh, nodeId, elementId):
    x = getNodalComponentFromOneElement(resultItem, mesh, nodeId, elementId, 'X')
    y = getNodalComponentFromOneElement(resultItem, mesh, nodeId, elementId, 'Y')
    z = getNodalComponentFromOneElement(resultItem, mesh, nodeId, elementId, 'Z')
    xy = getNodalComponentFromOneElement(resultItem, mesh, nodeId, elementId, 'XY')
    xz = getNodalComponentFromOneElement(resultItem, mesh, nodeId, elementId, 'XZ')
    yz = getNodalComponentFromOneElement(resultItem, mesh, nodeId, elementId, 'YZ')

    return [x,y,z,xy,xz,yz] 

#-----------------------------------------------------------------------------------
#	 gets one component values for nodes
#-----------------------------------------------------------------------------------
def getNodalComponentFromOneElement(resultItem, mesh, nodeId, elementId, rstComp):
    resultItem.SelectComponents([rstComp])
    nodalValue = [0.0,0.0,0.0]
    tensor = resultItem.GetElementValues(elementId)
    element = mesh.ElementById(elementId)
    indexOfNode = element.NodeIds.IndexOf(nodeId)
    itoadd = tensorLink[element.Type][indexOfNode]
    for i in itoadd:
        nodalValue[0] += tensor[i]/itoadd.Count
        nodalValue[1] += tensor[i+(len(tensor)/3)]/itoadd.Count
        nodalValue[2] += tensor[i+(2*(len(tensor))/3)]/itoadd.Count
    return nodalValue

#-----------------------------------------------------------------------------------
#	calcululates N for low cycle objects
#-----------------------------------------------------------------------------------
def calculateLifeLowCycleNormal(mesh, collector, resultSet, bodyData, N0):
	N = {}
	errorLog = {}
	combinedExcepter = 0
	combinedHighLife = 0
	excepter = 0
	for bodyId in bodyData:
		bodyElements = mesh.MeshRegionById(bodyId).ElementIds
		for elementId in resultSet:
			if elementId in bodyElements:
				elemN = []
				for i in xrange(len(resultSet[elementId])):
					if (resultSet[elementId][i]) == 0:
						elemN_temp = 10e9
						combinedHighLife += 1
						elemN.append(elemN_temp)
					else:
						elemN_temp, excepter, highLife, log = newtonRaphson(elementId, i, resultSet[elementId][i], bodyData[bodyId][1],bodyData[bodyId][2], bodyData[bodyId][3], N0)
						combinedExcepter += excepter
						combinedHighLife += highLife
						elemN.append(elemN_temp)
						errorLog[elementId, i] = log
				N[elementId] = elemN
	return N, combinedExcepter, combinedHighLife, errorLog		

#-----------------------------------------------------------------------------------
#	 calculates N for low cycle plane plated weld objects
#-----------------------------------------------------------------------------------
def calculateLowCyclePlanePlates(mesh, hotSpotValue, bodyData, N0):
	N = {}; errorLog = {}
	combinedExcepter = 0; combinedHighLife = 0; excepter = 0
	for bodyId in bodyData:
		emodule = bodyData[bodyId][1]
		strengthCoeff = bodyData[bodyId][2]
		ductilityCoeff = bodyData[bodyId][3]
		bodyNodeIds = mesh.MeshRegionById(bodyId).NodeIds 
		for nodeId in hotSpotValue:
			if nodeId in bodyNodeIds:
				N[nodeId] = []
				for i in xrange(len(hotSpotValue[nodeId])):
					if hotSpotValue[nodeId][i] == 0:
						combinedHighLife += 1
						N[nodeId].append(10e9) 
					else:	
						nodeN_temp, excepter, highLife, log = newtonRaphson(nodeId, i, hotSpotValue[nodeId][i], emodule, strengthCoeff, ductilityCoeff, float(N0))
						combinedExcepter += excepter 
						combinedHighLife += highLife
						N[nodeId].append(nodeN_temp)
						errorLog[nodeId, i] = log 
	return N, combinedExcepter, combinedHighLife, errorLog					

#-----------------------------------------------------------------------------------
#	 calculates N for high cycle plane plated weld objects
#-----------------------------------------------------------------------------------
def calculateHighCyclePlanePlates(mesh, hotSpotValue, bodyData):
	hotspotlogN = {}
	for bodyId in bodyData:
		environment = bodyData[bodyId][1]
		m1 = bodyData[bodyId][2]
		logA1 = bodyData[bodyId][3]
		m2 = bodyData[bodyId][4]
		logA2 = bodyData[bodyId][5]
		thicknessmodfactor = bodyData[bodyId][8]
		crackLimit = bodyData[bodyId][9]
		bodyNodes = mesh.MeshRegionById(bodyId).NodeIds
		snCurve = bodyData[bodyId][0]
		for nodeId in hotSpotValue:
			if nodeId in bodyNodes:
				hotspotlogN[nodeId] = []
				zeroSet = False
				for item in hotSpotValue[nodeId]:
					if item == 0:
						zeroSet = True
				if zeroSet == True:
					hotspotlogN[nodeId] = map(operator.add, hotSpotValue[nodeId], len(hotSpotValue[nodeId])*(307., ))
				else:
					#multiply thickmodfac med stress rang
					hotSpotValue[nodeId] = map(operator.mul, hotSpotValue[nodeId], len(hotSpotValue[nodeId])*(thicknessmodfactor, ))
					#check if m1 or m2 is to be used
					tempLogN = []
					for i in xrange(len(hotSpotValue[nodeId])):
						if stressSmallerThanLimit(hotSpotValue[nodeId][i], crackLimit) and snCurve != 'HighStrengthSteel' and environment != 'Seawater Free Corrosion':
							m = m2 
							logA = logA2
						else:
							m = m1
							logA = logA1
						#manipulate stressrange to log form		
						hotSpotValue[nodeId][i] = math.log10(hotSpotValue[nodeId][i])
						#log(stressrange)*m
						hotSpotValue[nodeId][i] = hotSpotValue[nodeId][i]*m
						#logN = loga - m logStressRange
						tempLogN.append(logA - hotSpotValue[nodeId][i])
					hotspotlogN[nodeId] = tempLogN
	return hotspotlogN

#-----------------------------------------------------------------------------------
#	 calculates N for high cycle  objects
#-----------------------------------------------------------------------------------
def calculateLifeHighCycleNormal(mesh, collector, resultSet, bodyData):
	logN = {}
	for bodyId in bodyData:
		bodyElements = mesh.MeshRegionById(bodyId).ElementIds
		for elementId in resultSet:
			if elementId in bodyElements: #and elementId not in logN:
				logN[elementId] = []
				zeroSet = False
				for item in resultSet[elementId]:
					if item == 0:
						zeroSet = True 
				if zeroSet == True:
					logN[elementId] = map(operator.add, resultSet[elementId], len(resultSet[elementId])*(307., ))
				else:
					#some bodydata info
					environment = bodyData[bodyId][1]
					snCurve = bodyData[bodyId][0]
					m1 = bodyData[bodyId][2]
					logA1 = bodyData[bodyId][3]
					m2 = bodyData[bodyId][4]
					logA2 = bodyData[bodyId][5]
					thicknessmodificationfactor = bodyData[bodyId][8]
					crackLimit = bodyData[bodyId][9]
					#multiply resultset with thicknessmodificationfactor
					resultSet[elementId] = map(operator.mul, resultSet[elementId], len(resultSet[elementId])*(thicknessmodificationfactor, ))
					#check if you need to use m1 or m2
					tempLogN = []
					for i in xrange(len(resultSet[elementId])):
						if stressSmallerThanLimit(resultSet[elementId][i], crackLimit) and snCurve != 'HighStrengthSteel':
							m = m2
							logA = logA2
						else:
							m = m1
							logA = logA1
						#manipulate stressrange to log10 form
						resultSet[elementId][i] = math.log10(resultSet[elementId][i])
						#resultset times m2
						resultSet[elementId][i] = resultSet[elementId][i]*m
						#do logN = logA2 - resulSet when resultset now = m2*log(deltaStress*thickmodfac)
						tempLogN.append(logA - resultSet[elementId][i])
					logN[elementId] = tempLogN
	return logN					

#-----------------------------------------------------------------------------------
#	 checks if stress range is over the limit to decide with logA and m to use
#-----------------------------------------------------------------------------------
def stressSmallerThanLimit(stressRange, limit):
	if stressRange < limit:
		return True
	else:
		return False	

#-----------------------------------------------------------------------------------
#	Calculate principal rst ranges based on all 6 rst components from t1 and t2
#-----------------------------------------------------------------------------------
def calculatePrincipal_FatigueDNV(resultComponentsT1, resultComponentsT2, Stress, stressOption):
	for i in xrange(len(resultComponentsT2)):
		try:
			resultComponentsT2[i] = map(operator.sub, resultComponentsT2[i], resultComponentsT1[i])
		except:
			resultComponentsT2[i] = resultComponentsT2[i]	

	x  = resultComponentsT2[0]
	y  = resultComponentsT2[1]
	z  = resultComponentsT2[2]
	xy = resultComponentsT2[3]
	xz = resultComponentsT2[4]
	yz = resultComponentsT2[5]


	main1 = []; main2 = []; main3 = []; resultSet = [];
	append1 = main1.append
	append2 = main2.append	
	append3 = main3.append
	if stressOption != 'Equivalent Stress':
		for (x,y,z,xy,xz,yz) in zip(x,y,z,xy,xz,yz):
			if Stress == False:
				principal = eigenValue_FatigueDNV(x,y,z,0.5*xy,0.5*xz,0.5*yz)
			else:
				principal = eigenValue_FatigueDNV(x,y,z,xy,xz,yz)	
			append1(principal[int('1')])
			append2(principal[int('2')])
			append3(principal[int('3')])
		main1 = map(abs, main1)
		main2 = map(abs, main2)
		main3 = map(abs, main3)	
		for i in xrange(len(main1)):
			resultSet.append(max(main1[i], main2[i], main3[i]))	
	else:
		for (x,y,z,xy,xz,yz) in zip(x,y,z,xy,xz,yz):
			equvNum = math.sqrt(0.5*((x-y)**2 + (y-z)**2 + (z-x)**2) + 3.*(xy**2 + yz**2 + xz**2))
			resultSet.append(equvNum)
	return resultSet	

#-----------------------------------------------------------------------------------
#	get all 6 strain components for elementId
#-----------------------------------------------------------------------------------
def getAllResultComponents_FatigueDNV(resultItem, Id, convUnit, plotOption, Shell):
	resultComponents = []; 
	resultComponents.append(getResultComponent_FatigueDNV(resultItem, 'X', Id, plotOption, Shell))
	resultComponents.append(getResultComponent_FatigueDNV(resultItem, 'Y', Id, plotOption, Shell))
	resultComponents.append(getResultComponent_FatigueDNV(resultItem, 'Z', Id, plotOption, Shell))
	resultComponents.append(getResultComponent_FatigueDNV(resultItem, 'XY', Id, plotOption, Shell))
	resultComponents.append(getResultComponent_FatigueDNV(resultItem, 'XZ', Id, plotOption, Shell))
	resultComponents.append(getResultComponent_FatigueDNV(resultItem, 'YZ', Id, plotOption, Shell))

	for i in xrange(len(resultComponents)):
		resultComponents[i] = map(operator.mul, resultComponents[i], len(resultComponents[i])*(convUnit,))

	return resultComponents

#-----------------------------------------------------------------------------------
#	get one strain component based on component name rstComp and elementId
#-----------------------------------------------------------------------------------
def getResultComponent_FatigueDNV(resultItem, rstComp, Id, plotOption, Shell):
	resultItem.SelectComponents([rstComp])
	components = resultItem.GetElementValues(Id)
	returnComponents = []
	if plotOption == 'Top' and Shell == True:	
		for i in xrange(len(components)/3,2*len(components)/3):
			returnComponents.append(components[i])
	elif plotOption == 'Middle' and Shell == True:
		for i in xrange(2*len(components)/3,len(components)):
			returnComponents.append(components[i])
	else:
		return resultItem.GetElementValues(Id)			

	return returnComponents

#-----------------------------------------------------------------------------------
#	find emodule based on material and temperature. Interpolates if necessary
#-----------------------------------------------------------------------------------
def findEmodule(userTemp, analysis, bodyId):
	mat = analysis.GeoData.GeoEntityById(bodyId).Material
	prop = materials.GetMaterialPropertyByName(mat,'Elasticity')
	tempList = prop["Temperature"][1:]
	emodList = prop["Young's Modulus"][1:]
	fromUnit = prop["Young's Modulus"][0]
	quantityName = 'Stress'
	toUnit = 'MPa'
	convUnit = units.ConvertUnit(1., fromUnit, toUnit, quantityName)

	#convert temp to suit the unit in tempLits
	tempListUnit = prop["Temperature"][0]
	userUnit = analysis.CurrentConsistentUnitFromQuantityName("Temperature")
	userTemp = units.ConvertUnit(userTemp, userUnit, tempListUnit, 'Temperature')

	if len(emodList) == 2:
		return (emodList[1]*convUnit)
	for i in xrange(1,len(tempList)):	
		if userTemp == tempList[i]:
			return (emodList[i]*convUnit)
	else:
		emod = materials.InterpolateData(tempList, emodList, userTemp)
		return (emod*convUnit)

#-----------------------------------------------------------------------------------
#	gets time to collect values based on timestep
#-----------------------------------------------------------------------------------
def getTimeFromStep(analysis, reader, timeStep):
    if timeStep == 'Last':
	    if  ((set(reader.ListTimeFreq) & set(analysis.StepsEndTime)) is not None):
	        timeStep = max(set(reader.ListTimeFreq) & set(analysis.StepsEndTime))
	    else:
	        
	        return -2               
    else:
        timeStep = int(float(timeStep))
        if timeStep == 0:
            timeStep = 0.
        else:
            timeStep = analysis.StepsEndTime[timeStep-1]
    return timeStep 

#-----------------------------------------------------------------------------------
#	writes to file when newton rapson did not converge
#-----------------------------------------------------------------------------------
def writeToFile(dictLog, analysis):
	#create file to hold error log in iteration 
	workDir = str(analysis.WorkingDir)
	errorObjectPath = os.path.join(workDir, "errorLogLowCycleFatigue.txt")
	if os.path.isfile(errorObjectPath):
		os.remove(errorObjectPath)

	errorObject = open(errorObjectPath, "a")
	#write to file
	
	for i in dictLog:
		try:
			errorObject.write("ElementId " + str(dictLog[i][0]) +" tensor index " + str(dictLog[i][1]) + " with stainrange " + str(dictLog[i][2]) + " did not converge after " + str(dictLog[i][5]) + " iterations." + '\n' )
			errorObject.write("This data can help in making decision if analysis is accurate enough: " + '\n')
			errorObject.write("		Cycles to Failure calculated " + str(dictLog[i][4]) + '\n')
			errorObject.write("		Differance between N1 and previously iterated N0 " + str(dictLog[i][3]) + '\n')
			errorObject.write("		Cycles to Failure function with N0 as input " + str(dictLog[i][6]) + '\n')
			errorObject.write("		Derivative cycles to failure function with N0 as input " + str(dictLog[i][7]) + '\n')
			errorObject.write("		If differance is sufficiently small, one can consider the results as satisfactorily." + '\n') 
			errorObject.write("		If the function or the Derivative of the funtions is zero or nan, it implies that the initial guess for Newton-Raphson method was too high. Try a lower number and retry." + '\n')
		except:
			pass	
	#close file
	errorObject.close()