#\file geometryHelpers.py
#
#----------------------------------------------------------------------------------
#
#   Implemented geometric methods for fatigue calculations
#   
#----------------------------------------------------------------------------------
#----------------------------------------------------------------------------------
#
# author: Tone Wermundsen
# Last update: 18.12.2014
# ansys v15 ACT 
#
#-----------------------------------------------------------------------------------



# def getTubularLocations(hotSpotId, mesh, hotspotData, directions, bodyData):
#     locations = {}
#     hsType = hotspotData[3]
#     hsBodies = hotspotData[2]
#     hotspot = mesh.NodeById(hotspotId)
#     hotspotLocation = [hotspot.X, hotspot.Y, hotspot.Z]
#     if hsType == 'Crown Point':
#         #myBrace = 
#         for bodyId in directions:
#             radius = bodyData[bodyId][11]
#             bodyType = bodyData[bodyId][14]
#             bodyThickness = bodyData[bodyId][10]
#             if bodyType == 'brace':
#                 a = 0.2*math.sqrt(radius*bodyThickness)
#                 b = 0.65*math.sqrt(radius*bodyThickness)
#             if bodyType == 'chord':
#                 a = 0.2*math.sqrt(radiusBRACE*thicknessBRACE)
#                 b = 0.4* math.sqrt(math.sqrt(radiusBRACE*thicknessBRACE*radius*thickness))
#     if hsType == 'Saddle Point':
#         for bodyId in directions:
#             radius = bodyData[bodyId][11]
#             bodyType = bodyData[bodyId][14]
#             bodyThickness = bodyData[bodyId][10]     
#             if bodyType == 'brace':
#                 a = 0.2*math.sqrt(radius*bodyThickness)
#                 b = 0.65*math.sqrt(radius*bodyThickness)
#             if bodyType == 'chord':    
#                 a = 0.2*math.sqrt(radiusBRACE*thicknessBRACE)
#                 b = math.pi*radius/36

# def getTubularExtDirections(myMesh, myGeometry, hotspot, hotspotData, bodyData):
#     directions = {}
#     hsType = hotspotData[3]
#     hsMainBody = hotspotData[4]
#     hsEdges = hotspotData[0]
#     hsFaces = hotspotData[1]
#     hsBodies = hotspotData[2]
#     if hsType == 'Crown Point':
#         #directions are the same as the directions of the connected bodies pipedirection
#         for bodyId in hsBodies:
#             directions[bodyId] = []
#             direction1 = bodyData[bodyId][12]
#             direction2 = map(operator.mul, len(direction1)*(-1, ), direction1)
#             directions[bodyId].append(direction1)
#             directions[bodyId].append(direction2)   
#     if hsType == 'Saddle Point':  
#         for bodyId in hsBodies:
#             directions[bodyId] = []
#             bodyType = bodyData[bodyId][14]
#             if bodyType == 'brace':
#                 #direction same as the direction of the connected body
#                 direction1 = bodyData[bodyId][12]
#                 direction2 = map(operator.mul, len(direction1)*(-1, ), direction1)
#                 directions[bodyId].append(direction1)
#                 directions[bodyId].append(direction2) 
#             elif bodyType == 'chord':
#                 #her snakker vi kjipt: her er retningen bestemt av center (rett under hs) og retningen til chorden, retningen vi regner med da er den 
#                 #retning som er vinkelrett paa chord retning og en vinkel alfa fra p til rettundersenter. dette finner vi ikke for vi har den faktiske
#                 #avstanden.. Eller, prove lage en retning som er avhengig av en vinkel
#                 direction1 = [0,0,0]
#                 direction2 = [-0,-0,-0]
#                 directions[bodyId].append(direction1)
#                 directions[bodyId].append(direction2)
#     return directions            

#----------------------------------------------------------------------------------- 
#   Finds point to extrapolate from on tubular joints
#-----------------------------------------------------------------------------------
def getTubularLocations(distanceClass, hotspotId, hotspotData, bodyData, mesh, geometry):
    locations = {}
    hotspotType = hotspotData[0]
    braceId = hotspotData[2]
    braceRadius = bodyData[braceId][11]
    braceThickness = bodyData[braceId][10]
    braceDirection = bodyData[braceId][12]
    chordId = hotspotData[1]
    chordRadius = bodyData[chordId][11]
    chordThickness = bodyData[chordId][10]
    chordDirection = bodyData[chordId][12]
    chordCircumference = bodyData[chordId][13]
    if hotspotType == 'Crown Point':
        braceDirection1 = braceDirection
        braceDirection2 = map(operator.mul, len(braceDirection)*(-1, ), braceDirection)
        chordDirection1 = chordDirection
        chordDirection2 = map(operator.mul, len(chordDirection)*(-1, ), chordDirection)
        if distanceClass == 'small':
            chordA = 0.2*math.sqrt(braceRadius*braceThickness)
            braceA = chordA
        else:
            chordA = 0.4*math.sqrt(math.sqrt(braceRadius*braceThickness*chordRadius*chordThickness))
            braceA = 0.65*math.sqrt(braceRadius*braceThickness)
        braceP1 = getExtCoordinatesFromOneDirection(mesh, hotspotId, braceDirection1, braceA)
        braceP2 = getExtCoordinatesFromOneDirection(mesh, hotspotId, braceDirection2, braceA)
        chordP1 = getExtCoordinatesFromOneDirection(mesh, hotspotId, chordDirection1, chordA)
        chordP2 = getExtCoordinatesFromOneDirection(mesh, hotspotId, chordDirection2, chordA)
        locations[chordId] = [chordP1, chordP2] 
        locations[braceId] = [braceP1, braceP2]
    elif hotspotType == 'Saddle Point':
        braceDirection1 = braceDirection
        braceDirection2 = map(operator.mul, len(braceDirection)*(-1, ), braceDirection)  
        if distanceClass == 'small':
            chordA = 0.2*math.sqrt(braceRadius*braceThickness)
            braceA = chordA
        else:
            chordA = math.pi*chordRadius/36
            braceA = 0.65*math.sqrt(braceRadius*braceThickness)
        braceP1 = getExtCoordinatesFromOneDirection(mesh, hotspotId, braceDirection1, braceA)
        braceP2 = getExtCoordinatesFromOneDirection(mesh, hotspotId, braceDirection2, braceA)    
        chordP1 = getExtCoordinatesAlongCurve(chordA, chordId, chordDirection, hotspotId, mesh, geometry, chordRadius, 1)
        chordP2 = getExtCoordinatesAlongCurve(chordA, chordId, chordDirection, hotspotId, mesh, geometry, chordRadius, 2)
        locations[chordId] = [chordP1, chordP2]
        locations[braceId] = [braceP1, braceP2]
    return locations        

#----------------------------------------------------------------------------------- 
#   Finds point to extrapolate from on a curve
#-----------------------------------------------------------------------------------
def getExtCoordinatesAlongCurve(chordA, chordId, chordDirection, hotspotId, mesh, geometry, chordRadius, number):
    hotspot = mesh.NodeById(hotspotId)
    hotSpotLocation = [hotspot.X, hotspot.Y, hotspot.Z]
    centerToHotSpot = findCenterToHotSpotDirection(chordId, hotSpotLocation, geometry, chordRadius)
    CP = map(operator.sub, hotSpotLocation, centerToHotSpot)
    angle = 360*chordA/(2*math.pi*chordRadius)
    if number == 2:
        angle = 360-angle
    v = rodriguesRotateVector(chordDirection, CP, angle)

    location = map(operator.add, v, centerToHotSpot)
    return location


#----------------------------------------------------------------------------------- 
#   Finds vector from a hot spot to a beams centerline
#-----------------------------------------------------------------------------------
def findCenterToHotSpotDirection(chordId, P, geometry, chordRadius):
    #start chordcentroid = A, chordcentroid = B, hotspoloct=P pointofind=c
    A = geometry.GeoEntityById(chordId).Centroid
    edges = geometry.GeoEntityById(chordId).Edges 
    for edge in edges:
        edgeId = edge.Id 
        if str(geometry.GeoEntityById(edgeId).CurveType) == 'GeoCurveCircle':
            B = geometry.GeoEntityById(edgeId).Centroid
    AB = map(operator.sub, B, A)
    AP = map(operator.sub, P, A)        
    normal = crossProduct(AB, AP)
    CP = crossProduct(normal, AB)
    CP = unitVector(CP)
    CP = map(operator.mul, CP, len(CP)*(chordRadius, ))
    C = map(operator.add, CP, P)
    return C         

#----------------------------------------------------------------------------------- 
#   classifies a hot spot
#-----------------------------------------------------------------------------------
def fillHotSpotData(allhotspots, result, table, mesh, geometry):
    hotspotData = {}
    for row in xrange(table.RowCount):
        table.ActiveRow = row 
        hotspotId = int(table.Properties["hotspotIdColumn"].ValueString)
        chordBodyId = int(table.Properties["chordBodyColumn"].Value)
        braceBodyId = int(table.Properties["braceBodyColumn"].Value)
        hotspot = mesh.NodeById(hotspotId)
        classification = table.Properties["classificationColumn"].ValueString
        hotspotData[hotspotId] = [classification, chordBodyId, braceBodyId]
    return hotspotData  

#----------------------------------------------------------------------------------- 
#   defines bodies from different geometry types
#-----------------------------------------------------------------------------------
def getBodyIds(result):
    geometry = ExtAPI.DataModel.GeoData
    mesh = result.Analysis.MeshData
    bodyIds = []
    try:
        geoIds = result.Properties["scope"].Properties["geometry"].Value.Ids 
    except:
        return bodyIds
    try:
        if str(result.Properties["scope"].Properties["geometry"].Value.SelectionType) == 'GeometryEntities':
            if geometry.GeoEntityById(geoIds[0]).Type.ToString() == GeoCellTypeEnum.GeoEdge.ToString() or geometry.GeoEntityById(geoIds[0]).Type.ToString() == GeoCellTypeEnum.GeoFace.ToString() or geometry.GeoEntityById(geoIds[0]).Type.ToString() == GeoCellTypeEnum.GeoVertex.ToString() : #input is edge, face or vertex
                for Id in geoIds:
                    for body in geometry.GeoEntityById(Id).Bodies:
                        if body.Id not in bodyIds:
                            bodyIds.append(body.Id)
            elif geometry.GeoEntityById(geoIds[0]).Type.ToString() == GeoCellTypeEnum.GeoBody.ToString():
                bodyIds = geoIds  
    except:
        MessageBox.Show(ExtAPI.UserInterface.MainWindow, "Unvalid input.", 'Error', MessageBoxType.Error, MessageBoxButtons.OK)             
    return bodyIds   

#----------------------------------------------------------------------------------- 
#   defines bodies from mesh-nodes
#-----------------------------------------------------------------------------------
def getBodyIds_FromNodes(geoIds, mesh, geometry):
	bodyIds = []
 	for i in xrange(len(geoIds)):
		hotspot = mesh.NodeById(geoIds[i])
		hsGeos = hotspot.GeoEntityIds
 		for geo in hsGeos:
 			if str(geometry.GeoEntityById(geo).Type) == "GeoBody":
				if geo not in bodyIds:   
					bodyIds.append(geo)
	return bodyIds				

#----------------------------------------------------------------------------------- 
#   gathers info about each load case in a result object
#-----------------------------------------------------------------------------------
def fillLoadCaseInfo(lcCombo, myAnalysis, myReader, result, resultView, lcTable):
	loadCase = {}
	if lcCombo == 'No':
		if resultView == 'Damage':
			numberOfCycles = result.Properties["loadCaseCombo"].Properties["numberOfCycles"].Value
		else:
			numberOfCycles = 0	
		startStep = result.Properties["loadCaseCombo"].Properties["startStep"].Value
		endStep = result.Properties["loadCaseCombo"].Properties["endStep"].Value
		t1 = getTimeFromStep(myAnalysis, myReader, startStep)
		t2 = getTimeFromStep(myAnalysis, myReader, endStep)
		loadCase[1] = [myAnalysis.Id, myAnalysis, float(t1), float(t2), int(numberOfCycles)]
	else:	
		for row in xrange(0,lcTable.RowCount):
			lcTable.ActiveRow = row
			lcAnalysisName = lcTable.Properties["loadCaseColumn"].ValueString
			for ans in ExtAPI.DataModel.AnalysisList:
				if ans.PhysicsType == 'Mechanical':
					if ans.Name == lcAnalysisName:
						lcAnalysisId = ans.Id 
						lcAnalysis = ans 
						lcReader = ans.GetResultsData()
						break
			lcStartStep = lcTable.Properties["startStepColumn"].Value
			lcEndStep = lcTable.Properties["endStepColumn"].Value
			lcStartStep = getTimeFromStep(ans, lcReader, lcStartStep)
			lcEndStep = getTimeFromStep(ans, lcReader, lcEndStep)
			if resultView == 'Damage':	
				lcNumberOfCycles = lcTable.Properties["numberOfCyclesColumn"].Value
			else:
				lcNumberOfCycles = 0	
			loadCase[row] = [lcAnalysisId, lcAnalysis, float(lcStartStep), float(lcEndStep), int(lcNumberOfCycles)]
			result.Attributes['Analysis_list'].append(ans.Id)
	return loadCase

#----------------------------------------------------------------------------------- 
#   find elements to extrapolate from in tubular joints when points are known
#-----------------------------------------------------------------------------------
def getTubularElements(mesh, hotspotId, locations, bodyData, geometry, geoUnit, meshUnit):
    hotspot = mesh.NodeById(hotspotId)
    elements = {}
    connectedElements = hotspot.ConnectedElementIds
    centroidFirstElement = mesh.ElementById(connectedElements[0]).Centroid
    startElementId = connectedElements[0]
    for bodyId in locations:
        elements[bodyId] = []
        rayStart = geometry.GeoEntityById(bodyId).Centroid
        rayStart[0] = units.ConvertUnit(rayStart[0], geoUnit, meshUnit, 'Length')
        rayStart[1] = units.ConvertUnit(rayStart[1], geoUnit, meshUnit, 'Length')
        rayStart[2] = units.ConvertUnit(rayStart[2], geoUnit, meshUnit, 'Length')
        for i in xrange(len(locations[bodyId])):
            thiselement = 0
            thisLocation = locations[bodyId][i]
            startDifferance = abs(centroidFirstElement[0]-thisLocation[0])+abs(centroidFirstElement[1]-thisLocation[1])+abs(centroidFirstElement[2]-thisLocation[2])
            for elementId in connectedElements:
                element = mesh.ElementById(elementId)
                centroid = element.Centroid
                newDifferance = abs(centroid[0]-thisLocation[0])+abs(centroid[1]-thisLocation[1])+abs(centroid[2]-thisLocation[2])
                if newDifferance < startDifferance:
                    startElementId = elementId
            rayLine =  [thisLocation[0]-rayStart[0], thisLocation[1]-rayStart[1], thisLocation[2]-rayStart[2]]
            negRayStart = [-1*rayStart[0], -1*rayStart[1], -1*rayStart[2]]
            #iteration constants
            iterCount = 0
            maxIter = 50
            searchedElements = []
            haveFoundElement = False
            while haveFoundElement == False:
                iterCount +=1
                if iterCount > maxIter:
                    elements[bodyId].append(-2)
                    break
                if startElementId == -2:
                    elements[bodyId].append(-2)
                    break
                intersectionCount = 0
                secondIntersectionCount = 0
                lines = defineElementLines(mesh, startElementId)    
                for j in xrange(len(lines)):
                    thisLine = lines[j]
                    if pointIsOnLine(mesh, thisLine, thisLocation):
                        elements[bodyId].append(startElementId)
                        haveFoundElement = True
                        break
                    else:
                        thisCount = rayCasting(mesh, thisLine, thisLine[3], thisLine[4], rayStart, thisLocation)
                        secondCount = rayCasting(mesh, thisLine, thisLine[3], thisLine[4], negRayStart, thisLocation)    
                        intersectionCount += thisCount  
                        secondIntersectionCount += secondCount
                if isOdd(intersectionCount) and isOdd(secondIntersectionCount) and haveFoundElement == False:
                    elements[bodyId].append(startElementId)
                    haveFoundElement = True 
                    break
                elif haveFoundElement == False:
                    searchedElements.append(startElementId)
                    previousElementsId = startElementId
                    startElementId = findNextElementId(mesh, previousElementsId, searchedElements, bodyId, thisLocation)   
    return elements 

#----------------------------------------------------------------------------------- 
#   find elements to extrapolate from in plane plates welds when points are known
#-----------------------------------------------------------------------------------
def getExtElements(mesh, hotspotId, extLocations, faceId, rayStart):
    elements = []
    hotspot = mesh.NodeById(hotspotId)
    connectedElements = hotspot.ConnectedElementIds
    centroidFirstElement = mesh.ElementById(connectedElements[0]).Centroid
    startElementId = connectedElements[0]
    for i in xrange(len(extLocations)):
        thisLocation = extLocations[i]
        startDifferance = abs(centroidFirstElement[0] - thisLocation[0]) + abs(centroidFirstElement[1] - thisLocation[1]) + abs(centroidFirstElement[2]- thisLocation[2])
        for elementId in connectedElements:
            element = mesh.ElementById(elementId)
            centroid = element.Centroid
            newDifferance = abs(centroid[0] -  thisLocation[0]) + abs(centroid[1] -  thisLocation[1]) + abs(centroid[2] -  thisLocation[2])
            if newDifferance < startDifferance:
                if elementId in mesh.MeshRegionById(faceId).ElementIds:
                    startElementId = elementId
        rayLine = [thisLocation[0] - rayStart[0], thisLocation[1] - rayStart[1], thisLocation[2] - rayStart[2]]
        negRayStart = [-1*rayStart[0], -1*rayStart[1], -1*rayStart[2]]
        #iteration constants
        iterCount = 0
        maxIter = 50
        searchedElements = []
        haveFoundElement = False
        while haveFoundElement == False:
            iterCount += 1
            if iterCount > maxIter:
                elements.append(-2)
                break
            if startElementId == -2:
                elements.append(startElementId)
                break
            intersectionCount = 0
            secondIntersectionCount = 0        
            lines = defineElementLines(mesh, startElementId)
            for j in xrange(len(lines)):
                #if point is on line:
                thisLine = lines[j]
                if pointIsOnLine(mesh, thisLine, thisLocation):
                    elements.append(startElementId)
                    haveFoundElement = True
                    break
                else:
                    thisCount = rayCasting(mesh, thisLine, thisLine[3], thisLine[4], rayStart, thisLocation)
                    secondCount = rayCasting(mesh, thisLine, thisLine[3], thisLine[4], negRayStart, thisLocation)    
                    intersectionCount += thisCount
                    secondIntersectionCount += secondCount
            if isOdd(intersectionCount) and isOdd(secondIntersectionCount) and haveFoundElement == False:
                elements.append(startElementId)
                haveFoundElement = True

                break
            elif haveFoundElement == False:
                searchedElements.append(startElementId)
                previousElementsId = startElementId
                startElementId = findNextElementId(mesh, previousElementsId, searchedElements, faceId, thisLocation)
    return elements  

#----------------------------------------------------------------------------------- 
#   finds next potential element to check if a point lies within its boundaries
#-----------------------------------------------------------------------------------
def findNextElementId(mesh, previousElementId, searchedElementIds, faceId, extLoc):
    #try to deside next element to search by asking for all potential nexts (all connected to the prev)
    #take away everything that has been searched
    #ask whitch centroid lies closest to extLoc and choose this one
    allPotentialElementIds = []
    cornerNodes = mesh.ElementById(previousElementId).CornerNodeIds
    for nodeId in cornerNodes:
        elementIds = mesh.NodeById(nodeId).ConnectedElementIds
        for elemId in elementIds:
            if elemId not in searchedElementIds:
                allPotentialElementIds.append(elemId)
    if len(allPotentialElementIds) == 0:
        return findRandomElementIdInFace(mesh, searchedElementIds, previousElementId, faceId)
    else:
        elem0Centroid = mesh.ElementById(allPotentialElementIds[0]).Centroid
        distance = math.sqrt((elem0Centroid[0]-extLoc[0])**2 + (elem0Centroid[1]-extLoc[1])**2 + (elem0Centroid[2]-extLoc[2])**2)
        nextElem = allPotentialElementIds[0]
        for elementId in allPotentialElementIds:
            centroid = mesh.ElementById(elementId).Centroid
            thisDistance = math.sqrt((centroid[0]-extLoc[0])**2 + (centroid[1]-extLoc[1])**2+(centroid[2]-extLoc[2])**2)
            if thisDistance < distance:
                #if elementId in mesh.MeshRegionById(faceId).ElementIds:
                distance = thisDistance
                nextElem = elementId
    return nextElem

#----------------------------------------------------------------------------------- 
#   returns a random element in a face
#-----------------------------------------------------------------------------------
def findRandomElementIdInFace(mesh, searchedElementIds, previousElementId, faceId):
    faceElementIds = mesh.MeshRegionById(faceId).ElementIds
    nextId = -2 
    for elemId in faceElementIds:
        if elemId != previousElementId:
            if elemId not in searchedElementIds:
                nextId = elemId
                break
    return nextId

#----------------------------------------------------------------------------------- 
#   defines lines of a element shape
#-----------------------------------------------------------------------------------
def defineElementLines(mesh, elementId):
    lines = []
    element = mesh.ElementById(elementId)
    nodes = element.CornerNodeIds
    for i in xrange(len(nodes)):
        nodeId1 = nodes[i]
        if i == (len(nodes)-1):
            nodeId2 = nodes[0]
        else:
            nodeId2 = nodes[i+1]
        node1 = mesh.NodeById(nodeId1)
        node2 = mesh.NodeById(nodeId2)

        if nodeId1 != nodeId2:
            line = [node2.X - node1.X, node2.Y - node1.Y, node2.Z - node1.Z, nodeId1, nodeId2]  
            lines.append(line)
        
    return lines   

 
#----------------------------------------------------------------------------------- 
#   gathers and returns all elements that are to be extraoplated from
#-----------------------------------------------------------------------------------
def getAllExtrapolationElements(smallExtElements, largeExtElements):
    elements = []
    for key in smallExtElements:
        for i in xrange(len(smallExtElements[key])):
            if smallExtElements[key][i] != -2:
                if smallExtElements[key][i] not in elements:
                    elements.append(smallExtElements[key][i])
            if largeExtElements[key][i] != -2:
                if largeExtElements[key][i] not in elements:
                    elements.append(largeExtElements[key][i])
    return elements   

#----------------------------------------------------------------------------------- 
#   returns a point based on distance and direction from another point
#-----------------------------------------------------------------------------------
def getExtCoordinatesFromOneDirection(mesh, hotSpotId, direction, thickness):
    hotspot = mesh.NodeById(hotSpotId)

    x = hotspot.X + thickness*direction[0]
    y = hotspot.Y + thickness*direction[1]
    z = hotspot.Z + thickness*direction[2]
    extPoint = [x,y,z]

    return extPoint

   

#----------------------------------------------------------------------------------- 
#   gets directions of extrapolation for plane plates
#-----------------------------------------------------------------------------------
def getExtDirections(mesh, geometry, hotspotId, faceId, edgeId, weldline):
    edge              = geometry.GeoEntityById(edgeId)
   # if edge.CurveType == 'GeoCurveCircle' or edge.CurveType == 'GeoCurveEllipseFull' or edge.CurveType == 'GeoCurveNoGeometry' or edge.CurveType == 'GoeCurveUnknown':
    try:
        endVertex     = edge.EndVertex.Id 
        startVertex   = edge.StartVertex.Id 
       
    #else:
    except: 
        endVertex     = -5
        startVertex   = -5
       
    extDirections     = []
    face              = geometry.GeoEntityById(faceId)
    normals           = face.Normals
    thisNormal        = [normals[0], normals[1], normals[2]] 
    direction1        = crossProduct(thisNormal, weldline)
    direction2        = map(operator.mul, direction1, len(direction1)*(-1, ))
    extDirections.append(unitVector(direction1))
    extDirections.append(unitVector(direction2))
    if (hotspotId in mesh.MeshRegionById(startVertex).NodeIds) or (hotspotId in mesh.MeshRegionById(endVertex).NodeIds):
        direction3 = map(operator.mul, weldline, len(weldline)*(-1, ))
        extDirections.append(unitVector(direction3))
    return extDirections    

#----------------------------------------------------------------------------------- 
#   defines edges as welds if the edge is connected to at least 2 bodies
#-----------------------------------------------------------------------------------
def findWelds(geoIds, geometry):
    welds = []
    if geometry.GeoEntityById(geoIds[0]).Type.ToString() == GeoCellTypeEnum.GeoEdge.ToString():
        welds = geoIds
    elif geometry.GeoEntityById(geoIds[0]).Type.ToString() == GeoCellTypeEnum.GeoBody.ToString() or geometry.GeoEntityById(geoIds[0]).Type.ToString() == GeoCellTypeEnum.GeoFace.ToString():
        for geoId in geoIds:
            for edge in geometry.GeoEntityById(geoId).Edges:
                if edge.Id not in welds:
                    if len(edge.Bodies) > 1:
                        welds.append(edge.Id)
    return welds 

#----------------------------------------------------------------------------------- 
#   defines all nodes in a weld as hotspots
#-----------------------------------------------------------------------------------
def findHotspots(welds, mesh):
    hotSpots = {}
    for edgeId in welds:
        hotSpots[edgeId] = mesh.MeshRegionById(edgeId).NodeIds
    return hotSpots   

#----------------------------------------------------------------------------------- 
#   collects all faces that are conneted to a certain edge
#-----------------------------------------------------------------------------------
def findFacesConnectedToEdge(welds, geometry):
    faces = {}
    for edgeId in welds:
        edgeFaces = geometry.GeoEntityById(edgeId).Faces
        faces[edgeId] = []
        for face in edgeFaces:
            faces[edgeId].append(face.Id)
    return faces  

#----------------------------------------------------------------------------------- 
#   removes potential extrapolation points if they do not belong to an element
#-----------------------------------------------------------------------------------
def eliminateOutOfBoundElements(smallExtrapolationElements, largeExtrapolationElements, myEdgeIds, myHotSpotIds, myEdgeFaceIds):
    for edgeId in myEdgeIds:
        for hotSpotId in myHotSpotIds[edgeId]:
            for faceId in myEdgeFaceIds[edgeId]:
                for i in xrange(len(smallExtrapolationElements[edgeId, faceId, hotSpotId])):
                    if smallExtrapolationElements[edgeId, faceId, hotSpotId][i] == -2:
                        largeExtrapolationElements[edgeId, faceId, hotSpotId][i] = -2
                    elif largeExtrapolationElements[edgeId, faceId, hotSpotId][i] == -2:
                        smallExtrapolationElements[edgeId, faceId, hotSpotId][i] = -2
                    else:
                        pass    