#\file lcTubular.py
#
#----------------------------------------------------------------------------------
#
#   Implemented controller for low cycle fatigue calculation on tubular joint 
#   
#----------------------------------------------------------------------------------
#----------------------------------------------------------------------------------
#
# author: Tone Wermundsen
# Last update: 18.12.2014
# ansys v15 ACT 
#
#-----------------------------------------------------------------------------------

#----------------------------------------------------------------------------------- 
#   Nested controller class that handles all evaluation behavior of the result object
#-----------------------------------------------------------------------------------
class LCTubularController():
	def __init__(self, ExtAPI, result):
		self.ExtAPI = ExtAPI
		self.result = result

	def evaluate(self, result, stepInfo, collector):
		result.Attributes['timeStamp'] = 0
		result.Attributes['Analysis_list'] = []
		ExtAPI.Log.WriteMessage("ready for LC tubular")
		MessageBox.Show(ExtAPI.UserInterface.MainWindow, "Unfinished result object, result may not apply or be viewed at all", 'Warning', MessageBoxType.Warning, MessageBoxButtons.OK) 
		#decs
		myAnalysis = result.Analysis 
		myMesh = myAnalysis.MeshData
		myGeometry = myAnalysis.GeoData
		myReader = myAnalysis.GetResultsData()


		#units
		meshUnit = myMesh.Unit 
		geoUnit = myGeometry.Assemblies[0].Unit
		userUnitTemp = myAnalysis.CurrentConsistentUnitFromQuantityName("Temperature")
		userUnitStress = myAnalysis.CurrentConsistentUnitFromQuantityName("Stress")
		userUnitLength = myAnalysis.CurrentConsistentUnitFromQuantityName("Length")

		#user input
		geoSelected = result.Properties["hotspotClassification"].Properties["geoSelected"].ValueString
		if geoSelected == 'No':
			msg = "Please select geometry and fill in classification table after selecting 'yes' at property Geometry is Selected"
			MessageBox.Show(ExtAPI.UserInterface.MainWindow, msg, 'Error', MessageBoxType.Error, MessageBoxButtons.OK)
			return
		classificationTable = result.Properties["hotspotClassification"].Properties["geoSelected"].Properties["classificationTable"]
		geoIds = result.Properties["scope"].Properties["geometry"].Value.Ids
		shellOption = result.Properties["scope"].Properties["shellOption"].ValueString
		resultView = result.Properties["resultView"].ValueString
		displayOption = result.Properties["integrationPointResult"].Properties["displayOption"].ValueString
		lcCombo = result.Properties["loadCaseCombo"].ValueString
		snEqual = result.Properties["snClass"].Properties["snEqual"].ValueString
		N0 = result.Properties["Advanced"].Properties["initialGuess"].Value
		lcTable = result.Properties["loadCaseCombo"].Properties["loadCaseTable"]

		bodyIds = getBodyIds_FromNodes(geoIds, myMesh, myGeometry)
		if len(bodyIds) == 0:
			MessageBox.Show(ExtAPI.UserInterface.MainWindow, "Unvalid input.", 'Error', MessageBoxType.Error, MessageBoxButtons.OK) 
			return
		bodyData = fillBodyDataLCTubular(bodyIds, result, snEqual, myGeometry, myAnalysis, userUnitLength, meshUnit)	
		myHotspots = geoIds
		if len(myHotspots) == 0:
			MessageBox.Show(ExtAPI.UserInterface.MainWindow, "Could not recognize any hot spots. Try changing geometry", 'Warning', MessageBoxType.Warning, MessageBoxButtons.OK) 
			return
		hotspotData = fillHotSpotData(myHotspots, result, classificationTable, myMesh, myGeometry)	
		loadCase = fillLoadCaseInfo(lcCombo, myAnalysis, myReader, result, resultView, lcTable)

		

		result.Attributes['timeStamp'] = time.time()

#----------------------------------------------------------------------------------- 
#   Fills bodyData with information about each bodies environment and material properties
#-----------------------------------------------------------------------------------
def fillBodyDataLCTubular(bodyIds, result, snEqual, geometry, analysis, userUnitLength, meshUnit):
	bodyData = {}
	if snEqual == 'Yes':
		userTemp = result.Properties["snClass"].Properties["snEqual"].Properties["temperature"].Value
		environment = result.Properties["snClass"].Properties["snEqual"].Properties["environment"].ValueString
		plateThickness = result.Properties["snClass"].Properties["snEqual"].Properties["plateThickness"].Value
		plateThickness = units.ConvertUnit(plateThickness, userUnitLength, meshUnit, "Length")
		strengthCoeff, ductilityCoeff = getStrengthAndDuctility(environment)
		for bodyId in bodyIds:
			thisMaterial = geometry.GeoEntityById(bodyId).Material
			emodule = findEmodule(userTemp, analysis, bodyId)
			bodyData[bodyId] = [environment, emodule, strengthCoeff, ductilityCoeff, plateThickness]
	else:
		table = result.Properties["snClass"].Properties["snEqual"].Properties["fatigueTable"]
		for row in xrange(table.RowCount):
			table.ActiveRow = row
			bodyId = bodyIds[row]
			environment = table.Properties["environmentColumn"].ValueString
			emodule = table.Properties["emoduleColumn"].Value
			plateThickness = table.Properties["plateThicknessColumn"].Value
			plateThickness = units.ConvertUnit(plateThickness, userUnitLength, meshUnit, "Length")
			strengthCoeff, ductilityCoeff = getStrengthAndDuctility(environment)
			bodyData[bodyId] = [environment, emodule, strengthCoeff, ductilityCoeff, plateThickness]
	return bodyData	