#\file snData.py
#
#----------------------------------------------------------------------------------
#
#	Implemented different S/E-N Curve data
#	
#----------------------------------------------------------------------------------
#----------------------------------------------------------------------------------
#
# author: Tone Wermundsen
# Last update:: 11.11.2014
# ansys v15 ACT
#
#-----------------------------------------------------------------------------------


#-----------------------------------------------------------------------------------
#	S-N Data class
#	  Input environment and S-N Curve Name from user input  	
# 	  returns m1, logA1, m2, logA2, stresslimit, k
#-----------------------------------------------------------------------------------


#returns m1, log a1, m2, loa2, stresslimit, k bortsett dea hos free corrosion der m, loga, k
def SNData_FatigueDNV(environment, snCurve):
	if snCurve == 'B1':
		if environment == 'Air': 								return 4.0, 15.117, 5.0, 17.146, 106.97, 0
		if environment == 'Seawater With Catodic Protection': 	return 4.0, 14.917, 5.0, 17.146, 106.97, 0
		if environment == 'Seawater Free Corrosion': 			return 3.0, 12.436, 3.0, 12.436, 0, 0
	if snCurve == 'B2':
		if environment == 'Air': 								return 4.0, 14.885, 5.0, 16.856, 93.59, 0
		if environment == 'Seawater With Catodic Protection': 	return 4.0, 14.685, 5.0, 16.856, 93.59, 0
		if environment == 'Seawater Free Corrosion': 			return 3.0, 12.262, 3.0, 12.262, 0, 0
	if snCurve == 'C':
		if environment == 'Air': 								return 3.0, 12.592, 5.0, 16.320, 73.10, 0.05
		if environment == 'Seawater With Catodic Protection': 	return 3.0, 12.192, 5.0, 16.320, 73.10, 0.05
		if environment == 'Seawater Free Corrosion': 			return 3.0, 12.115, 3.0, 12.115, 0, 0.15
	if snCurve == 'C1':
		if environment == 'Air': 								return 3.0, 12.449, 5.0, 16.081, 65.50, 0.10
		if environment == 'Seawater With Catodic Protection': 	return 3.0, 12.049, 5.0, 16.081, 65.50, 0.10
		if environment == 'Seawater Free Corrosion': 			return 3.0, 11.972, 3.0, 11.972, 0, 0.15
	if snCurve == 'C2':
		if environment == 'Air': 								return 3.0, 12.301, 5.0, 15.835, 58.48, 0.15
		if environment == 'Seawater With Catodic Protection': 	return 3.0, 11.901, 5.0, 15.385, 58.48, 0.15
		if environment == 'Seawater Free Corrosion': 			return 3.0, 11.824, 3.0, 11.824, 0, 0.15
	if snCurve == 'D':
		if environment == 'Air': 								return 3.0, 12.164, 5.0, 15.606, 52.63, 0.2
		if environment == 'Seawater With Catodic Protection': 	return 3.0, 11.764, 5.0, 15.606, 52.63, 0.2
		if environment == 'Seawater Free Corrosion': 			return 3.0, 11.687, 3.0, 11.687, 0, 0.2
	if snCurve == 'E':
		if environment == 'Air': 								return 3.0, 12.010, 5.0, 15.350, 46.78, 0.2
		if environment == 'Seawater With Catodic Protection': 	return 3.0, 11.610, 5.0, 15.350, 46.78, 0.2
		if environment == 'Seawater Free Corrosion': 			return 3.0, 11.533, 3.0, 11.533, 0, 0.2
	if snCurve == 'F':
		if environment == 'Air': 								return 3.0, 11.855, 5.0, 15.091, 41.52, 0.25
		if environment == 'Seawater With Catodic Protection': 	return 3.0, 11.455, 5.0, 15.091, 41.52, 0.25
		if environment == 'Seawater Free Corrosion': 			return 3.0, 11.378, 3.0, 11.378, 0, 0.25
	if snCurve == 'F1':
		if environment == 'Air': 								return 3.0, 11.699, 5.0, 14.832, 36.84, 0.25
		if environment == 'Seawater With Catodic Protection': 	return 3.0, 11.299, 5.0, 14.832, 36.84, 0.25
		if environment == 'Seawater Free Corrosion': 			return 3.0, 11.222, 3.0, 11.222, 0, 0.25
	if snCurve == 'F3':
		if environment == 'Air': 								return 3.0, 11.546, 5.0, 14.576, 32.75, 0.25
		if environment == 'Seawater With Catodic Protection': 	return 3.0, 11.146, 5.0, 14.576, 32.75, 0.25
		if environment == 'Seawater Free Corrosion': 			return 3.0, 11.068, 3.0, 11.068, 0, 0.25
	if snCurve == 'G':
		if environment == 'Air': 								return 3.0, 11.398, 5.0, 14.330, 29.24, 0.25
		if environment == 'Seawater With Catodic Protection': 	return 3.0, 10.998, 5.0, 14.330, 29.24, 0.25
		if environment == 'Seawater Free Corrosion': 			return 3.0, 10.921, 3.0, 10.921, 0, 0.25
	if snCurve == 'W1':
		if environment == 'Air': 								return 3.0, 11.261, 5.0, 14.101, 26.32, 0.25
		if environment == 'Seawater With Catodic Protection': 	return 3.0, 10.861, 5.0, 14.101, 26.32, 0.25
		if environment == 'Seawater Free Corrosion': 			return 3.0, 10.784, 3.0, 10.784, 0, 0.25
	if snCurve == 'W2':
		if environment == 'Air': 								return 3.0, 11.107, 5.0, 13.845, 23.39, 0.25
		if environment == 'Seawater With Catodic Protection': 	return 3.0, 10.707, 5.0, 13.845, 23.39, 0.25
		if environment == 'Seawater Free Corrosion': 			return 3.0, 10.630, 3.0, 10.630, 0, 0.25
	if snCurve == 'W3':
		if environment == 'Air': 								return 3.0, 10.970, 5.0, 13.617, 21.05, 0.25
		if environment == 'Seawater With Catodic Protection': 	return 3.0, 10.570, 5.0, 13.617, 21.05, 0.25
		if environment == 'Seawater Free Corrosion': 			return 3.0, 10.493, 3.0, 10.493, 0, 0.25
	if snCurve == 'T':	
		if environment == 'Air': 								return 3.0, 12.164, 5.0, 15.606, 52.63, 0.25 
		if environment == 'Seawater With Catodic Protection': 	return 3.0, 11.764, 5.0, 15.606, 52.63, 0.25
		if environment == 'Seawater Free Corrosion': 			return 3.0, 11.687, 3.0, 11.687, 0, 0.25
	if snCurve == 'HighStrengthSteel':
		if environment == 'Air':								return 4.7, 17.446, 4.7, 17.446, 235, 0 
		if environment == 'Seawater With Catodic Protection':   return 4.7, 17.446, 4.7, 17.446, 0, 0   


#-----------------------------------------------------------------------------------
#	E-N Data class
#	  Input environment from user input  	
# 	  returns strengthCoefficient, ductilityCoefficient
#-----------------------------------------------------------------------------------

def getStrengthAndDuctility(environment):
	if environment == 'Air':
		strength = 175
		duct = 0.091
	elif environment == 'Seawater With Cathodic Protection':
		strength = 160
		duct = 0.057	
	return strength, duct	